@@ -57,7 +57,7 @@ void LLVMToSPIRVDbgTran::transDebugMetadata() {
     if (!BM->isAllowedToUseVersion(VersionNumber::SPIRV_1_6))
       BM->addExtension(SPIRV::ExtensionID::SPV_KHR_non_semantic_info);
     else
-      BM->setMinSPIRVVersion(static_cast<SPIRVWord>(VersionNumber::SPIRV_1_6));
+      BM->setMinSPIRVVersion(VersionNumber::SPIRV_1_6);
   }
 
   for (DICompileUnit *CU : DIF.compile_units()) {
@@ -159,11 +159,9 @@ void LLVMToSPIRVDbgTran::finalizeDebugValue(
   SPIRVBasicBlock *BB = DV->getBasicBlock();
   Value *Val = DbgValue->getVariableLocationOp(0);
   DIExpression *Expr = DbgValue->getExpression();
-  if (!isNonSemanticDebugInfo()) {
-    if (DbgValue->getNumVariableLocationOps() > 1) {
-      Val = UndefValue::get(Val->getType());
-      Expr = DIExpression::get(M->getContext(), {});
-    }
+  if (DbgValue->getNumVariableLocationOps() > 1) {
+    Val = UndefValue::get(Val->getType());
+    Expr = DIExpression::get(M->getContext(), {});
   }
   using namespace SPIRVDebug::Operand::DebugValue;
   SPIRVWordVec Ops(MinOperandCount);
@@ -1292,7 +1290,13 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFunction(const DISubprogram *Func) {
       Ops.push_back(transDbgEntry(FuncDecl)->getId());
     else {
       Ops.push_back(getDebugInfoNoneId());
-      // no support for targetFuncName (Fortran trampoline) in LLVM 14
+      if (BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200) {
+        // Translate targetFuncName mostly for Fortran trampoline function if it
+        // is the case
+        StringRef TargetFunc = Func->getTargetFuncName();
+        if (!TargetFunc.empty())
+          Ops.push_back(BM->getString(TargetFunc.str())->getId());
+      }
     }
 
     DebugFunc = BM->addDebugInfo(SPIRVDebug::Function, getVoidTy(), Ops);
@@ -1462,7 +1466,7 @@ SPIRVExtInst *LLVMToSPIRVDbgTran::getSource(const T *DIEntry) {
                         ->getId());
     else if (BM->getDebugInfoEIS() ==
              SPIRVEIS_NonSemantic_Shader_DebugInfo_200) {
-      SPIRVDebug::FileChecksumKind ChecksumKind =
+      const SPIRVDebug::FileChecksumKind ChecksumKind =
           SPIRV::DbgChecksumKindMap::map(CheckSum.Kind);
 
       Ops.push_back(
@@ -1495,10 +1499,10 @@ SPIRVExtInst *LLVMToSPIRVDbgTran::getSource(const T *DIEntry) {
     if (NumWords < MaxNumWords)
       return Source;
 
-    uint64_t NumOfContinuedInstructions =
+    const uint64_t NumOfContinuedInstructions =
         NumWords / MaxNumWords - 1 + (NumWords % MaxNumWords ? 1 : 0);
     for (uint64_t J = 0; J < NumOfContinuedInstructions; J++) {
-      SPIRVWord Op = BM->getString(Str.substr(0, MaxStrSize))->getId();
+      const SPIRVWord Op = BM->getString(Str.substr(0, MaxStrSize))->getId();
       BM->addDebugInfo(SPIRVDebug::SourceContinued, getVoidTy(), {Op});
       Str.erase(0, MaxStrSize);
     }
