@@ -38,6 +38,11 @@
 ///
 //===----------------------------------------------------------------------===//
 
+// This file needs to be included before anything that declares
+// llvm::PointerType to avoid a compilation bug on MSVC.
+#include "llvm/Demangle/Demangle.h"
+#include "llvm/Demangle/ItaniumDemangle.h"
+
 #include "FunctionDescriptor.h"
 #include "ManglingUtils.h"
 #include "NameMangleAPI.h"
@@ -50,7 +55,6 @@
 
 #include "llvm/ADT/StringSwitch.h"
 #include "llvm/Bitcode/BitcodeWriter.h"
-#include "llvm/Demangle/Demangle.h"
 #include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/IntrinsicInst.h"
 #include "llvm/IR/Metadata.h"
@@ -345,6 +349,23 @@ bool isSYCLBfloat16Type(llvm::Type *Ty) {
   return false;
 }
 
+bool isLLVMCooperativeMatrixType(llvm::Type *Ty) {
+  // In LLVM 14/15, TargetExtType doesn't exist, so cooperative matrices are
+  // represented as struct types with names like
+  // "spirv.CooperativeMatrixKHR._..."
+  auto *ST = dyn_cast<StructType>(Ty);
+  if (!ST || !ST->hasName())
+    return false;
+
+  StringRef STName = ST->getName();
+  if (!STName.startswith(kSPIRVTypeName::PrefixAndDelim))
+    return false;
+
+  SmallVector<std::string, 8> Postfixes;
+  std::string TN = decodeSPIRVTypeName(STName, Postfixes);
+  return TN == kSPIRVTypeName::CooperativeMatrixKHR;
+}
+
 Function *getOrCreateFunction(Module *M, Type *RetTy, ArrayRef<Type *> ArgTypes,
                               StringRef Name, BuiltinFuncMangleInfo *Mangle,
                               AttributeList *Attrs, bool TakeName) {
@@ -544,7 +565,7 @@ bool getSPIRVBuiltin(const std::string &OrigName, spv::BuiltIn &B) {
   return getByName(R.str(), B);
 }
 
-// Demangled name is a substring of the name. The DemangledName is updated only
+// DemangledName is a substring of Name. The DemangledName is updated only
 // if true is returned
 bool oclIsBuiltin(StringRef Name, StringRef &DemangledName, bool IsCpp) {
   if (Name == "printf") {
@@ -583,6 +604,66 @@ bool oclIsBuiltin(StringRef Name, StringRef &DemangledName, bool IsCpp) {
   return true;
 }
 
+// DemangledName is a substring of Name. The DemangledName is updated only
+// if true is returned.
+bool isInternalSPIRVBuiltin(StringRef Name, std::string &DemangledName) {
+  if (!Name.startswith("_Z"))
+    return false;
+  constexpr unsigned DemangledNameLenStart = 2;
+  const size_t Start =
+      Name.find_first_not_of("0123456789", DemangledNameLenStart);
+  if (!Name.substr(Start).startswith(kSPIRVName::InternalBuiltinPrefix))
+    return false;
+  // LLVM 16 doesn't have itaniumDemangle(const char*, bool) overload
+  // Try using the C-style API first, but it may fail for types like bfloat
+  // (DF16b) that LLVM 16's demangler doesn't recognize
+  char *Demangled =
+      llvm::itaniumDemangle(Name.data(), nullptr, nullptr, nullptr);
+
+  if (Demangled) {
+    // Demangling succeeded - extract function name from demangled string
+    const std::string FullDemangled(Demangled);
+    std::free(Demangled);
+    if (!StringRef(FullDemangled).startswith(kSPIRVName::InternalBuiltinPrefix))
+      return false;
+    // Remove the __builtin_spirv_ prefix
+    const std::string WithoutPrefix =
+        FullDemangled.substr(strlen(kSPIRVName::InternalBuiltinPrefix));
+    // Extract just the function name (before the opening parenthesis)
+    const size_t ParenPos = WithoutPrefix.find('(');
+    if (ParenPos != std::string::npos)
+      DemangledName = WithoutPrefix.substr(0, ParenPos);
+    else
+      DemangledName = WithoutPrefix;
+  } else {
+    // Demangling failed (e.g., for bfloat types in LLVM 16)
+    // Fall back to extracting function name directly from the mangled name
+    // Format: _Z<len>__builtin_spirv_<FunctionName><TypeEncoding>
+    // All conversion function names end with "EXT" or "INTEL"
+    const StringRef AfterPrefix =
+        Name.substr(Start + strlen(kSPIRVName::InternalBuiltinPrefix));
+
+    const size_t ExtPos = AfterPrefix.find("EXT");
+    const size_t IntelPos = AfterPrefix.find("INTEL");
+    size_t FuncNameEnd = StringRef::npos;
+
+    if (ExtPos != StringRef::npos && IntelPos != StringRef::npos)
+      FuncNameEnd =
+          std::min(ExtPos + 3, IntelPos + 5); // +3 for "EXT", +5 for "INTEL"
+    else if (ExtPos != StringRef::npos)
+      FuncNameEnd = ExtPos + 3;
+    else if (IntelPos != StringRef::npos)
+      FuncNameEnd = IntelPos + 5;
+
+    if (FuncNameEnd == StringRef::npos)
+      DemangledName = AfterPrefix.str();
+    else
+      DemangledName = AfterPrefix.substr(0, FuncNameEnd).str();
+  }
+
+  return true;
+}
+
 // Check if a mangled type Name is unsigned
 bool isMangledTypeUnsigned(char Mangled) {
   return Mangled == 'h'    /* uchar */
